@@ -32,6 +32,7 @@ use std::{
     convert::{TryFrom, TryInto},
     str::FromStr,
 };
+use thiserror::Error;
 
 /// parse a Datalog fact
 pub fn fact(i: &str) -> IResult<&str, builder::Fact, Error> {
@@ -39,7 +40,7 @@ pub fn fact(i: &str) -> IResult<&str, builder::Fact, Error> {
 
     let (i, _) = error(
         preceded(space0, eof),
-        |input| format!("unexpected trailigng data after fact: '{}'", input),
+        |input| format!("unexpected trailing data after fact: '{}'", input),
         " ,\n",
     )(i)?;
 
@@ -605,7 +606,7 @@ fn integer(i: &str) -> IResult<&str, builder::Term, Error> {
 fn parse_date(i: &str) -> IResult<&str, u64, Error> {
     map_res(
         map_res(
-            take_while1(|c: char| c != ',' && c != ' ' && c != ')' && c != ']'),
+            take_while1(|c: char| c != ',' && c != ' ' && c != ')' && c != ']' && c != ';'),
             |s| chrono::DateTime::parse_from_rfc3339(s),
         ),
         |t| t.timestamp().try_into(),
@@ -837,7 +838,82 @@ pub fn parse_source(mut i: &str) -> Result<(&str, SourceResult), Vec<Error>> {
     }
 }
 
-#[derive(Debug, PartialEq)]
+pub fn parse_block_source(mut i: &str) -> Result<(&str, SourceResult), Vec<Error>> {
+    let mut result = SourceResult::default();
+    let mut errors = Vec::new();
+
+    loop {
+        if i.is_empty() {
+            if errors.is_empty() {
+                return Ok((i, result));
+            } else {
+                return Err(errors);
+            }
+        }
+
+        match terminated(
+            alt((
+                map(terminated(consumed(rule_inner), sep), |(i, r)| {
+                    SourceElement::Rule(i, r)
+                }),
+                map(terminated(consumed(fact_inner), sep), |(i, f)| {
+                    SourceElement::Fact(i, f)
+                }),
+                map(terminated(consumed(check_inner), sep), |(i, c)| {
+                    SourceElement::Check(i, c)
+                }),
+                map(line_comment, |_| SourceElement::Comment),
+                map(multiline_comment, |_| SourceElement::Comment),
+            )),
+            space0,
+        )(i)
+        {
+            Ok((i2, o)) => {
+                match o {
+                    SourceElement::Fact(i, f) => result.facts.push((i, f)),
+                    SourceElement::Rule(i, r) => result.rules.push((i, r)),
+                    SourceElement::Check(i, c) => result.checks.push((i, c)),
+                    SourceElement::Policy(_, _) => {},
+                    SourceElement::Comment => {}
+                }
+
+                i = i2;
+            }
+            Err(nom::Err::Incomplete(_)) => panic!(),
+            Err(nom::Err::Error(mut e)) => {
+                if let Some(index) = e.input.find(|c| c == ';') {
+                    e.input = &(e.input)[..index];
+                }
+
+                let offset = i.offset(e.input);
+                if let Some(index) = &i[offset..].find(|c| c == ';') {
+                    i = &i[offset + index + 1..];
+                } else {
+                    i = &i[i.len()..];
+                }
+
+                errors.push(e);
+            }
+            Err(nom::Err::Failure(mut e)) => {
+                if let Some(index) = e.input.find(|c| c == ';') {
+                    e.input = &(e.input)[..index];
+                }
+
+                let offset = i.offset(e.input);
+                if let Some(index) = &i[offset..].find(|c| c == ';') {
+                    i = &i[offset + index + 1..];
+                } else {
+                    i = &i[i.len()..];
+                }
+
+                errors.push(e);
+            }
+        }
+    }
+}
+
+#[derive(Error, Debug, PartialEq)]
+#[error("Parse error on input: {input}. Message: {message:?}")]
 pub struct Error<'a> {
     pub input: &'a str,
     pub code: ErrorKind,
@@ -1585,6 +1661,7 @@ mod tests {
             boolean, constrained_rule, fact, int, pred, rule, s, string, var, Binary, Check,
             Expression, Op, Policy, PolicyKind,
         };
+        use std::time::{Duration, SystemTime};
 
         let input = r#"
           fact("string", #symbol);
@@ -1605,7 +1682,9 @@ mod tests {
               fact(5678)
               or fact(1234), "test".starts_with("abc");
 
-          deny if true
+          check if 2021-01-01T00:00:00Z <= 2021-01-01T00:00:00Z;
+
+          deny if true;
         "#;
 
         let res = super::parse_source(input);
@@ -1664,6 +1743,24 @@ mod tests {
                     ),
                 ],
             },
+            Check {
+                queries: vec![constrained_rule(
+                    "query",
+                    empty_terms,
+                    empty_preds,
+                    &[Expression {
+                        ops: vec![
+                            Op::Value(builder::date(
+                                &(SystemTime::UNIX_EPOCH + Duration::from_secs(1609459200)),
+                            )),
+                            Op::Value(builder::date(
+                                &(SystemTime::UNIX_EPOCH + Duration::from_secs(1609459200)),
+                            )),
+                            Op::Binary(Binary::LessOrEqual),
+                        ],
+                    }],
+                )],
+            },
         ];
 
         let expected_policies = vec![
@@ -1711,4 +1808,123 @@ mod tests {
             expected_policies
         );
     }
+
+    #[test]
+    fn block_source_file() {
+        use builder::{
+            constrained_rule, fact, int, pred, rule, s, string, var, Binary, Check, Expression, Op,
+        };
+        use std::time::{Duration, SystemTime};
+
+        let input = r#"
+          fact("string", #symbol);
+          fact2(1234);
+
+          rule_head($var0) <- fact($var0, $var1), 1 < 2;
+
+          // line comment
+          check if 1 == 2;
+
+          /*
+           other comment
+          */
+
+          check if
+              fact(5678)
+              or fact(1234), "test".starts_with("abc");
+
+          check if 2021-01-01T00:00:00Z <= 2021-01-01T00:00:00Z;
+        "#;
+
+        let res = super::parse_block_source(input);
+        println!("parse_block_source res:\n{:#?}", res);
+
+        let empty_terms: &[builder::Term] = &[];
+        let empty_preds: &[builder::Predicate] = &[];
+
+        let expected_facts = vec![
+            fact("fact", &[string("string"), s("symbol")]),
+            fact("fact2", &[int(1234)]),
+        ];
+
+        let expected_rules = vec![constrained_rule(
+            "rule_head",
+            &[var("var0")],
+            &[pred("fact", &[var("var0"), var("var1")])],
+            &[Expression {
+                ops: vec![
+                    Op::Value(int(1)),
+                    Op::Value(int(2)),
+                    Op::Binary(Binary::LessThan),
+                ],
+            }],
+        )];
+
+        let expected_checks = vec![
+            Check {
+                queries: vec![constrained_rule(
+                    "query",
+                    empty_terms,
+                    empty_preds,
+                    &[Expression {
+                        ops: vec![
+                            Op::Value(int(1)),
+                            Op::Value(int(2)),
+                            Op::Binary(Binary::Equal),
+                        ],
+                    }],
+                )],
+            },
+            Check {
+                queries: vec![
+                    rule("query", empty_terms, &[pred("fact", &[int(5678)])]),
+                    constrained_rule(
+                        "query",
+                        empty_terms,
+                        &[pred("fact", &[int(1234)])],
+                        &[Expression {
+                            ops: vec![
+                                Op::Value(string("test")),
+                                Op::Value(string("abc")),
+                                Op::Binary(Binary::Prefix),
+                            ],
+                        }],
+                    ),
+                ],
+            },
+            Check {
+                queries: vec![constrained_rule(
+                    "query",
+                    empty_terms,
+                    empty_preds,
+                    &[Expression {
+                        ops: vec![
+                            Op::Value(builder::date(
+                                &(SystemTime::UNIX_EPOCH + Duration::from_secs(1609459200)),
+                            )),
+                            Op::Value(builder::date(
+                                &(SystemTime::UNIX_EPOCH + Duration::from_secs(1609459200)),
+                            )),
+                            Op::Binary(Binary::LessOrEqual),
+                        ],
+                    }],
+                )],
+            },
+        ];
+
+        let (_remaining, mut result) = res.unwrap();
+        //assert_eq!(remaining, "\n");
+        assert_eq!(
+            result.facts.drain(..).map(|(_, r)| r).collect::<Vec<_>>(),
+            expected_facts
+        );
+        assert_eq!(
+            result.rules.drain(..).map(|(_, r)| r).collect::<Vec<_>>(),
+            expected_rules
+        );
+        assert_eq!(
+            result.checks.drain(..).map(|(_, r)| r).collect::<Vec<_>>(),
+            expected_checks
+        );
+    }
 }
