@@ -537,10 +537,15 @@ impl Rule {
     }
 
     pub fn validate_variables(&self) -> Result<(), String> {
-        let mut head_variables: std::collections::HashSet<String> = self.0.ids.iter().filter_map(|term| match term {
-            Term::Variable(s) => Some(s.to_string()),
-            _ => None,
-        }).collect();
+        let mut head_variables: std::collections::HashSet<String> = self
+            .0
+            .ids
+            .iter()
+            .filter_map(|term| match term {
+                Term::Variable(s) => Some(s.to_string()),
+                _ => None,
+            })
+            .collect();
 
         for predicate in self.1.iter() {
             for term in predicate.ids.iter() {
@@ -550,15 +555,24 @@ impl Rule {
                         if head_variables.is_empty() {
                             return Ok(());
                         }
-                    },
-                    _ => {},
+                    }
+                    _ => {}
                 }
             }
         }
 
-        Err(format!("rule head contains variables that are not used in predicates of the rule's body: {}",
-                    head_variables.iter().map(|s| format!("${}", s)).collect::<Vec<_>>().join(", ")
-        ))
+        if head_variables.is_empty() {
+            Ok(())
+        } else {
+            Err(format!(
+                    "rule head contains variables that are not used in predicates of the rule's body: {}",
+                    head_variables
+                    .iter()
+                    .map(|s| format!("${}", s))
+                    .collect::<Vec<_>>()
+                    .join(", ")
+                    ))
+        }
     }
 }
 
