@@ -194,6 +194,20 @@ impl Biscuit {
         })
     }
 
+    /// deserializes a token and validates the signature using the root public key
+    pub fn from_base64<T: AsRef<[u8]>>(slice: T) -> Result<Self, error::Token> {
+        Biscuit::from_base64_with_symbols(slice, default_symbol_table())
+    }
+
+    /// deserializes a token and validates the signature using the root public key, with a custom symbol table
+    pub fn from_base64_with_symbols<T: AsRef<[u8]>>(
+        slice: T,
+        symbols: SymbolTable,
+    ) -> Result<Self, error::Token> {
+        let decoded = base64::decode_config(slice, base64::URL_SAFE)?;
+        Biscuit::from_with_symbols(&decoded, symbols)
+    }
+
     /// deserializes a sealed token and checks its signature with the secret, using a custom symbol table
     pub fn from_sealed(slice: &[u8], secret: &[u8]) -> Result<Self, error::Token> {
         Biscuit::from_sealed_with_symbols(slice, secret, default_symbol_table())
@@ -273,6 +287,16 @@ impl Biscuit {
         }
     }
 
+    pub fn to_base64(&self) -> Result<String, error::Token> {
+        match self.container.as_ref() {
+            None => Err(error::Token::InternalError),
+            Some(c) => c
+                .to_vec()
+                .map_err(error::Token::Format)
+                .map(|v| base64::encode_config(v, base64::URL_SAFE)),
+        }
+    }
+
     /// serializes the token
     pub fn serialized_size(&self) -> Result<usize, error::Token> {
         match self.container.as_ref() {
@@ -327,7 +351,7 @@ impl Biscuit {
         }
     }
 
-    pub(crate) fn generate_world(&self, symbols: &SymbolTable) -> Result<World, error::Logic> {
+    pub(crate) fn generate_world(&self, symbols: &mut SymbolTable) -> Result<World, error::Logic> {
         let mut world = World::new();
 
         let authority_index = symbols.get("authority").unwrap();
@@ -352,6 +376,15 @@ impl Biscuit {
             ));
         }
 
+        let mut unique_revocation_ids = self.unique_revocation_identifiers();
+        let unique_revocation_id_sym = symbols.insert("unique_revocation_id");
+        for (i, id) in unique_revocation_ids.drain(..).enumerate() {
+            world.facts.insert(Fact::new(
+                unique_revocation_id_sym,
+                &[ID::Integer(i as i64), ID::Bytes(id)],
+            ));
+        }
+
         for rule in self.authority.rules.iter().cloned() {
             if let Err(_message) = builder::Rule::convert_from(&rule, symbols).validate_variables() {
                 return Err(error::Logic::InvalidBlockRule(
@@ -441,8 +474,9 @@ impl Biscuit {
         verifier_checks: Vec<Check>,
         queries: HashMap<String, Rule>,
     ) -> Result<HashMap<String, Vec<Fact>>, error::Token> {
+        let mut symbols = symbols.clone();
         let mut world = self
-            .generate_world(symbols)
+            .generate_world(&mut symbols)
             .map_err(error::Token::FailedLogic)?;
 
         for fact in ambient_facts.drain(..) {
@@ -640,7 +674,10 @@ impl Biscuit {
         res
     }
 
-    /// returns a list of revocation Ids for each block, in order
+    /// returns a list of revocation identifiers for each block, in order
+    ///
+    /// if a token is generated with the same keys and the same content,
+    /// those identifiers will stay the same
     pub fn revocation_identifiers(&self) -> Vec<Vec<u8>> {
         use sha2::{Digest, Sha256};
 
@@ -666,6 +703,37 @@ impl Biscuit {
         res
     }
 
+    /// returns a list of unique revocation identifiers for each block, in order
+    ///
+    /// those identifiers will be different for every token even if they have the
+    /// same content and use the same keys
+    pub fn unique_revocation_identifiers(&self) -> Vec<Vec<u8>> {
+        use sha2::{Digest, Sha256};
+
+        let mut res = Vec::new();
+        let mut h = Sha256::new();
+
+        if let Some(token) = self.container.as_ref() {
+            h.update(&token.authority);
+            h.update(&token.keys[0].to_bytes());
+            h.update(&token.signature.parameters[0].compress().to_bytes());
+
+            let h2 = h.clone();
+            res.push(h2.finalize().as_slice().into());
+
+            for (i, block) in token.blocks.iter().enumerate() {
+                h.update(&block);
+                h.update(&token.keys[1 + i].to_bytes());
+                h.update(&token.signature.parameters[1 + i].compress().to_bytes());
+
+                let h2 = h.clone();
+                res.push(h2.finalize().as_slice().into());
+            }
+        }
+
+        res
+    }
+
     /// pretty printer for this token
     pub fn print(&self) -> String {
         let authority = print_block(&self.symbols, &self.authority);
@@ -682,6 +750,46 @@ impl Biscuit {
             blocks.join(",\n\t")
         )
     }
+
+    /// prints the content of a block as Datalog source code
+    pub fn print_block_source(&self, index: usize) -> Option<String> {
+        let block = if index == 0 {
+            &self.authority
+        } else {
+            match self.blocks.get(index - 1) {
+                None => return None,
+                Some(block) => block,
+            }
+        };
+
+        let facts: Vec<_> = block.facts.iter().map(|f| self.symbols.print_fact(f)).collect();
+        let rules: Vec<_> = block.rules.iter().map(|r| self.symbols.print_rule(r)).collect();
+        let checks: Vec<_> = block
+            .checks
+            .iter()
+            .map(|r| self.symbols.print_check(r))
+            .collect();
+
+        let mut res = facts.join(";\n");
+        if !facts.is_empty() {
+            res.push_str(";\n");
+        }
+        res.push_str(&rules.join(";\n"));
+        if !rules.is_empty() {
+            res.push_str(";\n");
+        }
+        res.push_str(&checks.join(";\n"));
+        if !checks.is_empty() {
+            res.push_str(";\n");
+        }
+
+        Some(res)
+    }
+
+    /// returns the number of blocks (at least 1)
+    pub fn block_count(&self) -> usize {
+        1 + self.blocks.len()
+    }
 }
 
 fn print_block(symbols: &SymbolTable, block: &Block) -> String {
