@@ -30,8 +30,8 @@ pub struct Verifier {
 
 impl Verifier {
     pub(crate) fn from_token(token: &Biscuit) -> Result<Self, error::Logic> {
-        let world = token.generate_world(&token.symbols)?;
-        let symbols = token.symbols.clone();
+        let mut symbols = token.symbols.clone();
+        let world = token.generate_world(&mut symbols)?;
 
         Ok(Verifier {
             world,
@@ -170,6 +170,15 @@ impl Verifier {
             ));
         }
 
+        let mut unique_revocation_ids = token.unique_revocation_identifiers();
+        let unique_revocation_id_sym = self.symbols.insert("unique_revocation_id");
+        for (i, id) in unique_revocation_ids.drain(..).enumerate() {
+            self.world.facts.insert(datalog::Fact::new(
+                unique_revocation_id_sym,
+                &[datalog::ID::Integer(i as i64), datalog::ID::Bytes(id)],
+            ));
+        }
+
         for rule in token.authority.rules.iter().cloned() {
             let r = Rule::convert_from(&rule, &token.symbols);
             let rule = r.convert(&mut self.symbols);
@@ -518,7 +527,7 @@ impl Verifier {
     }
 
     /// returns all of the data loaded in the verifier
-    pub fn dump(&self) -> (Vec<Fact>, Vec<Rule>, Vec<Check>) {
+    pub fn dump(&self) -> (Vec<Fact>, Vec<Rule>, Vec<Rule>, Vec<Check>, Vec<Policy>) {
         let mut checks = self.checks.clone();
         checks.extend(
             self.token_checks
@@ -538,7 +547,13 @@ impl Verifier {
                 .iter()
                 .map(|r| Rule::convert_from(r, &self.symbols))
                 .collect(),
+            self.world
+                .privileged_rules
+                .iter()
+                .map(|r| Rule::convert_from(r, &self.symbols))
+                .collect(),
             checks,
+            self.policies.clone(),
         )
     }
 }
